@@ -18,7 +18,7 @@ from .qualdr import QualDR, QualDR_Grading
 
 # Chest X-Ray
 from .chexpert import CheXpert, CheXpert_Small
-from .padchest import PadChest
+# from .padchest import PadChest
 
 # Microscopy
 from .BBBC038v1_microscopy import BBBC038v1
