@@ -35,7 +35,8 @@ class PreProcess(Dataset):
         return len(self._dset)
 
     def __iter__(self):
-        return iter(self._dset)
+        for x in self._dset:
+            yield self._fn(x)
 
     def __getattr__(self, x):
         """Pass-through to the wrapped dataset"""
