@@ -297,7 +297,7 @@ def f1_score(cm, beta=1, mode:str='both'):
     p = precision(cm, mode)
     r = recall(cm, mode)
     b = beta**2
-    return (1+b) * p*r / ( b * (p+r))
+    return ((1+b) * p*r / (b * (p+r))).nan_to_num_()
 
 
 def distance_correlation(x:torch.Tensor, y:torch.Tensor=None):
