@@ -53,7 +53,7 @@ def get_run_ids(ns):
 def load_df_from_fp(fp, ns):
     print('found data', fp)
     try:
-        if fp.endswith('.csv'):
+        if fp.endswith('.csv') or fp.endswith('.csv.gz'):
             df = pd.read_csv(fp).set_index(ns.index_col)
         elif fp.endswith('.h5'):
             df = pd.read_hdf(fp, ns.hdf_table)
@@ -110,7 +110,7 @@ def _get_subplots(N:int, sharey:bool):
 
 
 def _subplots_add_legend(fig, handles_labels):
-    print(handles_labels)
+    # print(handles_labels)
     fig.tight_layout()
     fig.subplots_adjust(bottom=0.1)
     # choose just one axis as the legend, since they are all identical
@@ -156,7 +156,7 @@ def make_plots(ns, cdfs, subplots:bool):
     if subplots:
         if ns.legend:
             _subplots_add_legend(fig, ax.get_legend_handles_labels())
-        yield (fig, col)
+        yield (fig, ns.col_regex)
 
 
 def savefig_with_symlink(fig, fp, symlink_fp):
@@ -197,6 +197,7 @@ def main(ns):
             '\n'.join([x for x in locals().keys() if not x.startswith('__')]),
             ', '.join(globals().keys())))
         IPython.start_ipython(['--no-banner'] + ipython_start_cmd, user_ns=dct)
+    return cdfs
 
 
 def mode_0(ns):
@@ -217,9 +218,8 @@ def mode_1_plots(ns):
     cdfs = _mode_1_and_4_get_perf_data_as_df(ns).reset_index('filename', drop=True)
 
     timestamp = dt.datetime.utcnow().strftime('%Y%m%dT%H%M%S.%f')  # date plot was created.  nothing to do with timestamp column.
-    os.makedirs(join(ns.mode1_savefig_dir, 'archive'), exist_ok=True)
-    for fig, col in make_plots(ns, cdfs, ns.mode1_subplots):
-        if ns.savefig and ns.mode1_subplots:
+    for fig, col in make_plots(ns, cdfs, ns.make_subplots):
+        if ns.savefig and ns.make_subplots:
             # would need, I think, to make each column a subfigure instead of
             # axes plot.
             warnings.warn(
@@ -228,8 +228,8 @@ def mode_1_plots(ns):
         if not ns.savefig: continue
         savefig_with_symlink(
             fig,
-            f'{ns.mode1_savefig_dir}/archive/{col}_{timestamp}.png',
-            f'{ns.mode1_savefig_dir}/{col}_latest.png')
+            f'{ns.savefig_dir}/{ns.runid_regex}/mode1_{timestamp}__{col}.png',
+            f'{ns.savefig_dir}/{ns.runid_regex}/mode1_latest__{col}.png')
     return cdfs
 
 
@@ -254,15 +254,14 @@ def mode_2_plots(ns):
         )
         cdfs_mode2[run_id] = cdfs
 
-        os.makedirs(f'{ns.mode2_savefig_dir}/archive'.format(run_id=run_id), exist_ok=True)
-        for fig, col in make_plots(ns, cdfs.reset_index('run_id', drop=True), False):
+        for fig, col in make_plots(ns, cdfs.reset_index('run_id', drop=True), ns.make_subplots):
             fig.suptitle(run_id)
             if not ns.savefig: continue
             # save to file
             savefig_with_symlink(
                 fig,
-                f'{ns.mode2_savefig_dir}/archive/{col}_{timestamp}.png'.format(run_id=run_id),
-                f'{ns.mode2_savefig_dir}/{col}_latest.png'.format(run_id=run_id))
+                f'{ns.savefig_dir}/{run_id}/mode2_{timestamp}__{col}.png'.format(run_id=run_id),
+                f'{ns.savefig_dir}/{run_id}/mode2_latest__{col}.png'.format(run_id=run_id))
     return cdfs_mode2
 
 
@@ -295,20 +294,38 @@ def mode_3_plots(ns):
 
     timestamp = dt.datetime.utcnow().strftime('%Y%m%dT%H%M%S.%f')  # date plot was created.  nothing to do with timestamp column.
     plot_cols = [prettify(col) for col in cols]
-    for col in plot_cols:
-        fig, ax = plt.subplots(1,1, figsize=(12,10), sharey=bool(ns.ylim))
+
+    if ns.make_subplots:
+        fig, axs = _get_subplots(len(plot_cols), sharey=bool(ns.ylim))
+    for n, col in enumerate(plot_cols):
+        if ns.make_subplots:
+            ax = axs.ravel()[n]
+        else:
+            fig, ax = plt.subplots(1,1)
         kws = dict(x=prettify(ns.index_col), y=col, hue='Run Id', ax=ax, data=cdfs)
-        sns.lineplot(**kws, ci=ns.mode3_ci or None, lw=2 if ns.mode3_ci == 0 else 1, legend=ns.legend)
+        z = sns.lineplot(**kws, errorbar=('ci', ns.mode3_ci) if ns.mode3_ci else None, lw=2 if ns.mode3_ci == 0 else 1, legend=ns.legend)
         if ns.mode3_ci == 0:
             sns.lineplot(**kws, units="Filename", estimator=None, lw=1, alpha=.5)#, legend=False)
         ax.set_title(col)
         if ns.ylim:
             ax.set_ylim(*ns.ylim)
-        if not ns.savefig: continue
-        savefig_with_symlink(
-            fig,
-            f'{ns.mode3_savefig_dir}/archive/{col}_mode3_{timestamp}.png',
-            f'{ns.mode3_savefig_dir}/{col}_mode3_latest.png')
+        if ns.legend and ns.make_subplots:
+            _handles, _labels = ax.get_legend_handles_labels()
+            ax.get_legend().remove()
+            # note: the legend labels may be incorrect if --mode4-groupby is not
+            # default value.
+            _subplots_add_legend(fig, ax.get_legend_handles_labels())
+
+        if ns.savefig and not ns.make_subplots:
+            savefig_with_symlink(
+                fig,
+                f'{ns.savefig_dir}/{ns.runid_regex}/mode3_{timestamp}__{col}.png',
+                f'{ns.savefig_dir}/{ns.runid_regex}/mode3_latest__{col}.png')
+    if ns.savefig and ns.make_subplots:
+            savefig_with_symlink(
+                fig,
+                f'{ns.savefig_dir}/{ns.runid_regex}/mode3_{timestamp}__{ns.col_regex}.png',
+                f'{ns.savefig_dir}/{ns.runid_regex}/mode3_latest__{ns.col_regex}.png')
     return cdfs_unmodified
 
 
@@ -368,7 +385,7 @@ def _mode4_make_subplots(grp:pd.DataFrame, ns):
             ax.set_title(f'{col}, SKIPPED because no numeric data')
             continue
         z.plot(ax=ax, title=subplot_title)
-        print(ns)
+        # print(ns)
         if ns.legend:
             _handles, _labels = ax.get_legend_handles_labels()
         ax.get_legend().remove()
@@ -388,8 +405,8 @@ def _mode4_make_subplots(grp:pd.DataFrame, ns):
         timestamp = dt.datetime.utcnow().strftime('%Y%m%dT%H%M%S')  # date plot was created.  nothing to do with timestamp column.
         savefig_with_symlink(
             fig,
-            f'{ns.mode4_savefig_dir}/archive/{run_id}_mode4_{timestamp}.png',
-            f'{ns.mode4_savefig_dir}/{run_id}_mode4_latest.png')
+            f'{ns.savefig_dir}/{run_id}/mode4_{timestamp}__{ns.col_regex}.png',
+            f'{ns.savefig_dir}/{run_id}/mode4_latest__{ns.col_regex}.png')
 
 
 def mode_4_plots(ns):
@@ -418,17 +435,57 @@ def mode_4_plots(ns):
 
 def bap():
     class F(ap.ArgumentDefaultsHelpFormatter, ap.RawTextHelpFormatter): pass
-    par = ap.ArgumentParser(formatter_class=F)
+    d = ('''
+Plot results of experiments. Expect a default directory structure like:
+
+        ./results/               # this default location is adjustable via --data-dir
+            experiment_1a/
+                log/perf*.csv    # used by --mode 2 and --mode 3
+                                 # to compare multiple csv files within this experiment
+
+                perf.csv         # used by --mode 1 and --mode 4
+                                 # to compare one csv file representing each experiment
+                                 # default filename is adjustable via --data-fp-regex.
+            experiment_1b/
+                perf.csv
+            another_experiment/
+                perf.csv
+            ...
+
+    where the CSV files should share the same header, and the header must
+    include a column "epoch" (adjustable via --index-col).
+
+    For example:
+      - One has a history of runs of the form "results/experiment_1a/log/perf_{timestamp}.csv"
+      - And a symlink 'results/experiment_1a/perf.csv -> log/perf_{latest_timestamp}.csv'
+    For example:
+      - One does K-fold cross validation and gets K csv files:  "results/experiment_1a/log/perf_{kfold_split_idx}.csv"
+      - And a single perf.csv file summarizing a model evaluated on an out-of-distribution test dataset "results/experiment_1a/perf.csv"
+
+Example command-line usage:
+
+    # Plot "epoch vs {XYZ}" for data from experiment_1a and experiment_1b,
+    # where {XYZ} is any column matching "Loss".
+
+    python -m simplepytorch.plot_perf  'experiment_1' -c 'Loss'
+
+    # note: the --mode option is very useful.
+
+    ''')
+    par = ap.ArgumentParser(description=d, formatter_class=F)
     A = par.add_argument
-    A('runid_regex', help='find the run_ids that match the regular expression.')
+    A('runid_regex', help='find the run_ids matching the regular expression. run_id is the identifier for an experiment containing data')
     A('--data-dir', default='./results', help=' ')
-    A('--data-fp-regex', default=r'perf.*\.csv$', help=' ')
+    A('--data-fp-regex', default=r'perf.*\.csv(\.gz)?$', help=dedent('''\
+    only consider data files matching the specified criteria:
+    ./{data_dir}/{runid_regex}/{data_fp_regex}.'''))
     #  A('--', nargs='?', default='one', const='two')
     A('--rolling-mean', '--rm', type=int, help='smoothing')
     A('--ylim', nargs=2, type=float, help='Optional. Two numbers denoting the the y limit shared by all plots. Example: --ylim 0 1')
     A('--hdf-table-name', help='required if searching .h5 files')
     A('-c', '--col-regex', default='^(?!epoch|batch_idx|timestamp)', help='plot only columns matching regex.  By default, plot all except epoch and batch_idx.')
-    A('--index-col', default='epoch', help=' ')
+    A('--index-col', default='epoch',
+      help='The x-axis of the plots.  All data files must contain this column.')
     A('--mode', default=1, type=int, choices=[0,1,2,3,4], help=dedent('''\
         `--mode 0` Don't plot anything, just collect data and drop into a shell.
         `--mode 1` Compare across experiments, with one plot per column (i.e. performance metric)
@@ -437,13 +494,11 @@ def bap():
                    confidence interval to consider history of runs for each run_id.
         `--mode 4` Compare across columns, one plot per experiment.
                    '''))
-    A('--mode1-savefig-dir', default='./results/plots/mode1', help=' ')
-    A('--mode2-savefig-dir', default='./results/plots/mode2', help=' ')
-    A('--mode3-savefig-dir', default='./results/plots/mode3', help=' ')
-    A('--mode4-savefig-dir', default='./results/plots/mode4', help=' ')
+    A('--savefig-dir', default='./results/plots/',
+      help='Default location to save figures if --savefig is defined')
     A('--mode3-ci', type=float,
       help='confidence interval for error bars.  If 0, also shows all lines individually')
-    A('--mode1-figures', dest='mode1_subplots', action='store_false', help=dedent('''\
+    A('--make-figures', dest='make_subplots', action='store_false', help=dedent('''\
         If defined, generate individual figures instead of a figure with subplots.'''))
     A('--mode4-groupby', default="(Train|Val|Test) (.*):2", help=dedent('''\
         Optionally, make one sub-plot for each group of columns. To use, pass a
@@ -462,7 +517,9 @@ def bap():
         Example:  `--mode4-allruns 2` just shows the last two runs.'''))
     A('--best-effort', action='store_true',
       help=" Try to load a csv file, but don't raise error if cannot read a file.")
-    A('--savefig', action='store_true', help="save plots to file")
+    A('--savefig', action='store_true', help=dedent('''\
+        Save plots to file.  Can be useful with --no-plot --no-shell.
+        Also can be useful with --make-figures.'''))
     A('--no-plot', action='store_true', help="if supplied, don't show the plots")
     A('--no-shell', action='store_false', dest='shell',
       help='do not drop into an IPython shell when finished')
