@@ -242,7 +242,7 @@ class MultiClassClassification(Result):
     """
     @property
     def metrics(self):
-        metrics = ('Loss', 'Num Samples', 'MCC', 'Acc', 'BAcc', 'cm', 'y', 'yhat', 'ROC_AUC')
+        metrics = ('Loss', 'Num Samples', 'MCC', 'Acc', 'BAcc', 'cm', 'ROC_AUC') # 'y', 'yhat',
         if self._num_classes == 2:
             metrics += ('Precision', 'Recall', 'F1', )
         return metrics
@@ -309,7 +309,7 @@ class MultiClassClassification(Result):
             return np.array([])
         if len(self._ydata) > 1:
             self._ydata = [T.cat(self._ydata, 0)]
-        return self._ydata[0].cpu().numpy()
+        return self._ydata[0].cpu().numpy().tolist()
 
     @property
     def yhat(self):
@@ -317,17 +317,17 @@ class MultiClassClassification(Result):
             return np.array([])
         if len(self._yhatdata) > 1:
             self._yhatdata = [T.cat(self._yhatdata, 0)]
-        return self._yhatdata[0].cpu().numpy()
+        return self._yhatdata[0].cpu().numpy().tolist()
 
     @property
     def roc_auc(self):
-        if self.y.shape[0] == 0:
+        if len(self.y) == 0:
             return 0 #np.array([])
         else:
-            y_score = self.yhat
+            y_score = np.array(self.yhat)
             multi_class = 'ovo' if y_score.shape[1] <= 25 else 'ovr'
             return roc_auc_score(
-                y_true=self.y, y_score=y_score, multi_class=multi_class)
+                y_true=np.array(self.y), y_score=y_score, multi_class=multi_class)
 
     def update(self, yhat, y, loss) -> None:
         """Compute and store results given:
@@ -364,7 +364,7 @@ class MultiLabelBinaryClassification(Result):
     This class is useful to to aggregate results, for instance over the course
     of an epoch.
     """
-    ALL_METRICS = ('Loss', 'Num Samples', 'MCC', 'Precision', 'Recall', 'F1', 'Acc', 'BAcc', 'cm', 'y', 'yhat', 'ROC_AUC')
+    ALL_METRICS = ('Loss', 'Num Samples', 'MCC', 'Precision', 'Recall', 'F1', 'Acc', 'BAcc', 'cm', 'ROC_AUC')  # 'y', 'yhat',
 
     @property
     def metrics(self):
@@ -452,24 +452,22 @@ class MultiLabelBinaryClassification(Result):
     def y(self):
         self._ydata = {k: [T.cat(v, 0)] if len(v) > 1 else v
                        for k, v in self._ydata.items()}
-        return {f'y {class_name}': v[0].cpu().numpy() if len(v) else np.array([])
+        return {f'y {class_name}': v[0].cpu().numpy().tolist() if len(v) else np.array([])
                 for class_name, v in self._ydata.items()}
 
     @property
     def yhat(self):
         self._yhatdata = {k: [T.cat(v, 0)] if len(v) > 1 else v
                           for k, v in self._yhatdata.items()}
-        return {f'yhat {class_name}': v[0].cpu().numpy() if len(v) else np.array([])
+        return {f'yhat {class_name}': v[0].cpu().numpy().tolist() if len(v) else np.array([])
                 for class_name, v in self._yhatdata.items()}
 
     @property
     def roc_auc(self):
-        y = self.y
-        yhat = self.yhat
         rv = {}
-        for kls, yval, yhatval in zip(self._cms, y.values(), yhat.values()):
+        for kls, yval, yhatval in zip(self._cms, self.y.values(), self.yhat.values()):
             key = f'ROC_AUC {kls}'
-            if yval.shape[0] <= 0:
+            if len(yval) <= 0:
                 rv[key] = 0
                 continue
             try:
